@@ -326,15 +326,16 @@ _wrap_vte_terminal_set_color_background(PyGObject *self, PyObject *args, PyObjec
     return Py_None;
 }
 
+#line 308 "vte.override"
 static PyObject *
 _wrap_vte_terminal_set_colors(PyGObject *self, PyObject *args, PyObject *kwargs)
 {
-    static char *kwlist[] = { "foreground", "background", "palette", "palette_size", NULL };
-    PyObject *py_foreground, *py_background, *py_palette;
-    int palette_size;
+    static char *kwlist[] = { "foreground", "background", "palette", NULL };
+    PyObject *py_foreground, *py_background, *py_palette, *item;
+    int palette_size, i;
     GdkColor *foreground = NULL, *background = NULL, *palette = NULL;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OOOi:VteTerminal.set_colors", kwlist, &py_foreground, &py_background, &py_palette, &palette_size))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OOO:VteTerminal.set_colors", kwlist, &py_foreground, &py_background, &py_palette, &palette_size))
         return NULL;
     if (pyg_boxed_check(py_foreground, GDK_TYPE_COLOR))
         foreground = pyg_boxed_get(py_foreground, GdkColor);
@@ -348,16 +349,30 @@ _wrap_vte_terminal_set_colors(PyGObject *self, PyObject *args, PyObject *kwargs)
         PyErr_SetString(PyExc_TypeError, "background should be a GdkColor");
         return NULL;
     }
-    if (pyg_boxed_check(py_palette, GDK_TYPE_COLOR))
-        palette = pyg_boxed_get(py_palette, GdkColor);
-    else {
-        PyErr_SetString(PyExc_TypeError, "palette should be a GdkColor");
+    if (PySequence_Check(py_palette)) {
+	palette_size = PySequence_Length(py_palette);
+        palette = g_malloc(sizeof(GdkColor) * palette_size);
+	for (i = 0; i < palette_size; i++) {
+	    item = PySequence_GetItem(py_palette, i);
+            if (!pyg_boxed_check(item, GDK_TYPE_COLOR)) {
+		g_free(palette);
+        	PyErr_SetString(PyExc_TypeError, "palette should be a list of GdkColors");
+		return NULL;
+	    }
+	    palette[i] = *((GdkColor*)pyg_boxed_get(py_palette, GdkColor));
+            Py_DECREF(item);
+	}
+    } else {
+        PyErr_SetString(PyExc_TypeError, "palette should be a list of GdkColors");
         return NULL;
     }
     vte_terminal_set_colors(VTE_TERMINAL(self->obj), foreground, background, palette, palette_size);
+    g_free(palette);
     Py_INCREF(Py_None);
     return Py_None;
 }
+#line 375 "vte.c"
+
 
 static PyObject *
 _wrap_vte_terminal_set_default_colors(PyGObject *self)
@@ -763,7 +778,7 @@ _wrap_vte_terminal_get_text(PyGObject *self, PyObject *args, PyObject *kwargs)
     	return Py_BuildValue("O", text);
     }
 }
-#line 767 "vte.c"
+#line 782 "vte.c"
 
 
 #line 213 "vte.override"
@@ -860,7 +875,7 @@ _wrap_vte_terminal_get_text_range(PyGObject *self, PyObject *args, PyObject *kwa
     	return Py_BuildValue("O", text);
     }
 }
-#line 864 "vte.c"
+#line 879 "vte.c"
 
 
 static PyObject *
@@ -1199,6 +1214,6 @@ pyvte_register_classes(PyObject *d)
     }
 
 
-#line 1203 "vte.c"
+#line 1218 "vte.c"
     pygobject_register_class(d, "VteTerminal", VTE_TYPE_TERMINAL, &PyVteTerminal_Type, Py_BuildValue("(O)", &PyGtkWidget_Type));
 }
